import logging
from datetime import datetime

import scrapy

from opennem.pipelines.files import LinkExtract
from opennem.pipelines.nem import ExtractCSV
from opennem.pipelines.wem.balancing_summary import WemStoreBalancingSummary
from opennem.spiders.dirlisting import DirlistingSpider


def get_date_component(format_str):
    return datetime.now().strftime(format_str)


class WemCurrentSpider(scrapy.Spider):

    url_params = {
        "day": get_date_component("%d"),
        "month": get_date_component("%m"),
        "year": get_date_component("%Y"),
    }

    def start_requests(self):
        request_url = self.start_url.format(**self.url_params)

        yield scrapy.Request(request_url)

    def parse(self, response):
        yield {"content": response.text}


class WemHistoricSpider(DirlistingSpider):
    allowed_domains = ["wa.nemweb.com.au"]
    pipelines = set([LinkExtract,])

    # Archives tend to contain large zips of embedded zips so throttle
    # to limit memory use
    custom_settings = {
        "CONCURRENT_REQUESTS": 4,
        "CONCURRENT_ITEMS": 4,
    }



import csv
import logging
from datetime import datetime

from scrapy.exceptions import DropItem
from sqlalchemy.exc import IntegrityError
from sqlalchemy.orm import sessionmaker

from opennem.db import db_connect
from opennem.db.models.bom import BomObservation
from opennem.utils.pipelines import check_spider_pipeline

logger = logging.getLogger(__name__)


def parse_date(date_str):
    dt = datetime.strptime(date_str, "%Y%m%d%H%M%S")

    return dt


class DatabaseStoreBase(object):
    def __init__(self):
        engine = db_connect()
        self.session = sessionmaker(bind=engine)


class StoreBomObservation(DatabaseStoreBase):
    @check_spider_pipeline
    def process_item(self, item, spider):

        s = self.session()

        print(item)

        observation = BomObservation(
            station_id=spider.observatory_id,
            observation_time=parse_date(item["aifstime_utc"]),
            temp_apparent=item["apparent_t"],
            temp_air=item["air_temp"],
            press_qnh=item["press_qnh"],
            wind_dir=item["wind_dir"],
            wind_spd=item["wind_spd_kmh"],
        )

        try:
            s.add(observation)
            s.commit()
        except Exception as e:
            logger.error("Error: {}".format(e))
        finally:
            s.close()

        return item



import csv
import logging

from scrapy.exceptions import DropItem
from sqlalchemy.exc import IntegrityError
from sqlalchemy.orm import sessionmaker

from opennem.db import db_connect
from opennem.db.models.opennem import NemDispatchPrice, NemDispatchUnitScada
from opennem.utils.pipelines import check_spider_pipeline

logger = logging.getLogger(__name__)

TABLE_MAP = {
    "DISPATCH_UNIT_SCADA": NemDispatchUnitScada,
    "DISPATCH_PRICE": NemDispatchPrice,
}


class ExtractCSV(object):
    @check_spider_pipeline
    def process_item(self, item, spider):

        if not "content" in item:
            return item

        content = item["content"]
        del item["content"]

        item["tables"] = []
        table = {"name": None}

        datacsv = csv.reader(content.split("\n"))

        for row in datacsv:
            if not row or type(row) is not list or len(row) < 1:
                continue

            record_type = row[0]

            if record_type == "C":
                # @TODO csv meta stored in table
                if table["name"] is not None:
                    item["tables"].append(table)

            elif record_type == "I":
                if table["name"] is not None:
                    item["tables"].append(table)

                table = {}
                table["name"] = "{}_{}".format(row[1], row[2])
                table["fields"] = fields = row[4:]
                table["records"] = []

            elif record_type == "D":
                values = row[4:]
                record = dict(zip(table["fields"], values))

                table["records"].append(record)

        return item


class DatabaseStore(object):
    def __init__(self):
        engine = db_connect()
        self.session = sessionmaker(bind=engine)

    @check_spider_pipeline
    def process_item(self, item, spider):

        s = self.session()

        if not "tables" in item:
            logger.info("No tables in item to process")
            return item

        tables = item["tables"]

        for table in tables:

            if not "name" in table:
                logger.error("No name in table")
                continue

            table_name = table["name"]

            if not table_name in TABLE_MAP:
                logger.error(
                    "Could not map table name to model for item {}".format(
                        table_name
                    )
                )
                continue

            Model = TABLE_MAP[table_name]

            if not "records" in table:
                logger.info(
                    "No records for item with table name {}".format(table_name)
                )
                continue

            records = table["records"]

            for record in records:

                i = Model(**record)

                try:
                    s.add(i)
                    s.commit()
                except IntegrityError:
                    pass
                except Exception as e:
                    s.rollback()
                    logger.error("Database error: {}".format(e))
                    logger.debug(i.__dict__)
                finally:
                    s.close()

        s.close()

        return item



"""
    settings files - read settings from env and from config.yml



"""


import os
from urllib.parse import urlparse

MODULE_DIR = os.path.dirname(__file__)

MYSQL_HOST_URL = os.getenv("MYSQL_HOST_URL", default=False)

DATABASE_HOST_URL = os.getenv("DATABASE_HOST_URL", default=False)

REDIS_HOST_URL = os.getenv("REDIS_HOST_URL", default=False)


def get_mysql_host(db_name=None):
    """
        read the database connection url from env and optionally replace the database name

    """
    db_url = None

    if not MYSQL_HOST_URL:
        raise Exception("MYSQL_HOST_URL not defined")

    db_url = MYSQL_HOST_URL

    if not db_name:
        return db_url

    db_url = replace_database_in_url(db_url, db_name)

    return db_url


def get_database_host():
    if DATABASE_HOST_URL:
        return DATABASE_HOST_URL
    raise Exception("DATABASE_HOST_URL not defined")


def get_redis_host():
    if REDIS_HOST_URL:
        return REDIS_HOST_URL
    raise Exception("REDIS_HOST_URL not defined")


def replace_database_in_url(db_url, db_name):
    """
        replaces the database portion of a database connection URL with db_name

        @param db_name database name to replace with
    """

    db_url_parsed = urlparse(db_url)
    db_url_parsed = db_url_parsed._replace(path=f"/{db_name}")

    return db_url_parsed.geturl()



"""
    OpenNEM primary schema adapted to support multiple energy sources

    Currently supported:

    - NEM
    - WEM
"""

from sqlalchemy import (
    NUMERIC,
    Boolean,
    Column,
    Date,
    DateTime,
    ForeignKey,
    Index,
    Integer,
    Numeric,
    Sequence,
    String,
    Table,
    Text,
    Time,
)
from sqlalchemy.dialects import postgresql
from sqlalchemy.ext.declarative import declarative_base
from sqlalchemy.orm import backref, relationship
from sqlalchemy.sql import func

Base = declarative_base()
metadata = Base.metadata


class NemModel(object):
    """
        Base model for both NEM and WEM

        Each table has an overrid for update and additional meta fields

        @TODO - upsert support for postgresql and mysql dialects (see db/__init__)
    """

    def update(self, **kwargs):
        for key, value in kwargs.items():
            if key == "id" or key.endswith("_id"):
                continue

            # @TODO watch how we do updates so we don't overwrite data
            cur_val = getattr(self, key)
            if key not in [None, ""]:
                setattr(self, key, value)

    def __compile_query(self, query):
        """Via http://nicolascadou.com/blog/2014/01/printing-actual-sqlalchemy-queries"""
        compiler = (
            query.compile
            if not hasattr(query, "statement")
            else query.statement.compile
        )
        return compiler(dialect=postgresql.dialect())

    def __upsert(
        self,
        session,
        model,
        rows,
        as_of_date_col="report_date",
        no_update_cols=[],
    ):
        table = model.__table__

        stmt = self.insert(table).values(rows)

        update_cols = [
            c.name
            for c in table.c
            if c not in list(table.primary_key.columns)
            and c.name not in no_update_cols
        ]

        on_conflict_stmt = stmt.on_conflict_do_update(
            index_elements=table.primary_key.columns,
            set_={k: getattr(stmt.excluded, k) for k in update_cols},
            index_where=(model.report_date < stmt.excluded.report_date),
        )

        print(self.compile_query(on_conflict_stmt))
        session.execute(on_conflict_stmt)

    created_at = Column(DateTime(timezone=True), server_default=func.now())
    updated_at = Column(DateTime(timezone=True), onupdate=func.now())


class NemDispatchUnitScada(Base, NemModel):
    __tablename__ = "nem_dispatch_unit_scada"
    __table_args__ = (
        Index(
            "nem_dispatch_unit_scada_uniq",
            "SETTLEMENTDATE",
            "DUID",
            unique=True,
        ),
    )

    # @TODO id columns are all being removed and replaced with composite primaries
    # only used in dev
    id = Column(Integer, primary_key=True)

    SETTLEMENTDATE = Column(DateTime, index=True)
    DUID = Column(Text, index=True,)
    SCADAVALUE = Column(NUMERIC(10, 6))


class NemDispatchCaseSolution(Base, NemModel):
    __tablename__ = "nem_dispatch_case_solution"
    __table_args__ = (
        Index(
            "nem_dispatch_case_solution_uniq", "SETTLEMENTDATE", unique=True,
        ),
    )

    id = Column(Integer, primary_key=True)
    SETTLEMENTDATE = Column(DateTime, index=True)
    RUNO = Column(Integer, nullable=False)
    INTERVENTION = Column(Integer)
    CASESUBTYPE = Column(Integer)
    SOLUTIONSTATUS = Column(Integer)
    SPDVERSION = Column(Integer)
    NONPHYSICALLOSSES = Column(Integer)
    TOTALOBJECTIVE = Column(Numeric)
    TOTALAREAGENVIOLATION = Column(Integer)
    TOTALINTERCONNECTORVIOLATION = Column(Integer)
    TOTALGENERICVIOLATION = Column(Integer)
    TOTALRAMPRATEVIOLATION = Column(Integer)
    TOTALUNITMWCAPACITYVIOLATION = Column(Integer)
    TOTAL5MINVIOLATION = Column(Integer)
    TOTALREGVIOLATION = Column(Integer)
    TOTAL6SECVIOLATION = Column(Integer)
    TOTAL60SECVIOLATION = Column(Integer)
    TOTALASPROFILEVIOLATION = Column(Integer)
    TOTALFASTSTARTVIOLATION = Column(Integer)
    TOTALENERGYOFFERVIOLATION = Column(Integer)
    LASTCHANGED = Column(Integer)
    SWITCHRUNINITIALSTATUS = Column(Integer)
    SWITCHRUNBESTSTATUS = Column(Integer)
    SWITCHRUNBESTSTATUS_INT = Column(Integer)


class NemDispatchPrice(Base, NemModel):
    __tablename__ = "nem_dispatch_price"
    __table_args__ = (
        Index(
            "nem_dispatch_price_uniq", "SETTLEMENTDATE", "DUID", unique=True,
        ),
    )

    id = Column(Integer, primary_key=True)
    SETTLEMENTDATE = Column(DateTime, index=True)
    DUID = Column(Text, index=True,)
    SCADAVALUE = Column(NUMERIC(10, 6))

    REGIONID = Column(Text, index=True,)
    RUNO = Column(Integer, nullable=False)

    DISPATCHINTERVAL = Column(Integer, nullable=False)
    INTERVENTION = Column(Integer, nullable=False)
    RRP = Column(Integer, nullable=False)
    EEP = Column(Integer, nullable=False)
    ROP = Column(Integer, nullable=False)
    APCFLAG = Column(Integer, nullable=False)
    MARKETSUSPENDEDFLAG = Column(Integer, nullable=False)
    LASTCHANGED = Column(DateTime(timezone=True), nullable=False)
    RAISE6SECRRP = Column(Integer, nullable=False)
    RAISE6SECROP = Column(Integer, nullable=False)
    RAISE6SECAPCFLAG = Column(Integer, nullable=False)
    RAISE60SECRRP = Column(Integer, nullable=False)
    RAISE60SECROP = Column(Integer, nullable=False)
    RAISE60SECAPCFLAG = Column(Integer, nullable=False)
    RAISE5MINRRP = Column(Integer, nullable=False)
    RAISE5MINROP = Column(Integer, nullable=False)
    RAISE5MINAPCFLAG = Column(Integer, nullable=False)
    RAISEREGRRP = Column(Integer, nullable=False)
    RAISEREGROP = Column(Integer, nullable=False)
    RAISEREGAPCFLAG = Column(Integer, nullable=False)
    LOWER6SECRRP = Column(Integer, nullable=False)
    LOWER6SECROP = Column(Integer, nullable=False)
    LOWER6SECAPCFLAG = Column(Integer, nullable=False)
    LOWER60SECRRP = Column(Integer, nullable=False)
    LOWER60SECROP = Column(Integer, nullable=False)
    LOWER60SECAPCFLAG = Column(Integer, nullable=False)
    LOWER5MINRRP = Column(Integer, nullable=False)
    LOWER5MINROP = Column(Integer, nullable=False)
    LOWER5MINAPCFLAG = Column(Integer, nullable=False)
    LOWERREGRRP = Column(Integer, nullable=False)
    LOWERREGROP = Column(Integer, nullable=False)
    LOWERREGAPCFLAG = Column(Integer, nullable=False)
    PRICE_STATUS = Column(Integer, nullable=False)
    PRE_AP_ENERGY_PRICE = Column(Integer, nullable=False)
    PRE_AP_RAISE6_PRICE = Column(Integer, nullable=False)
    PRE_AP_RAISE60_PRICE = Column(Integer, nullable=False)
    PRE_AP_RAISE5MIN_PRICE = Column(Integer, nullable=False)
    PRE_AP_RAISEREG_PRICE = Column(Integer, nullable=False)
    PRE_AP_LOWER6_PRICE = Column(Integer, nullable=False)
    PRE_AP_LOWER60_PRICE = Column(Integer, nullable=False)
    PRE_AP_LOWER5MIN_PRICE = Column(Integer, nullable=False)
    PRE_AP_LOWERREG_PRICE = Column(Integer, nullable=False)
    CUMUL_PRE_AP_ENERGY_PRICE = Column(Integer, nullable=False)
    CUMUL_PRE_AP_RAISE6_PRICE = Column(Integer, nullable=False)
    CUMUL_PRE_AP_RAISE60_PRICE = Column(Integer, nullable=False)
    CUMUL_PRE_AP_RAISE5MIN_PRICE = Column(Integer, nullable=False)
    CUMUL_PRE_AP_RAISEREG_PRICE = Column(Integer, nullable=False)
    CUMUL_PRE_AP_LOWER6_PRICE = Column(Integer, nullable=False)
    CUMUL_PRE_AP_LOWER60_PRICE = Column(Integer, nullable=False)
    CUMUL_PRE_AP_LOWER5MIN_PRICE = Column(Integer, nullable=False)
    CUMUL_PRE_AP_LOWERREG_PRICE = Column(Integer, nullable=False)

    # @TODO make enum
    OCD_STATUS = Column(Text, nullable=False)
    MII_STATUS = Column(Text, nullable=False)


class NemDispatchRegionSum(Base, NemModel):
    __tablename__ = "nem_dispatch_region_sum"
    __table_args__ = (
        # Index("nem_dispatch_region_sum_uniq", "SETTLEMENTDATE", unique=True,),
    )

    SETTLEMENTDATE = Column(Integer, nullable=False, primary_key=True)
    RUNNO = Column(Integer, nullable=False)
    REGIONID = Column(Text, nullable=False, primary_key=True)
    DISPATCHINTERVAL = Column(
        DateTime(timezone=True), nullable=False, primary_key=True
    )
    INTERVENTION = Column(Integer, nullable=False)
    TOTALDEMAND = Column(Numeric, nullable=False)
    AVAILABLEGENERATION = Column(Numeric, nullable=True)
    AVAILABLELOAD = Column(Numeric, nullable=True)
    DEMANDFORECAST = Column(Numeric, nullable=True)
    DISPATCHABLEGENERATION = Column(Numeric, nullable=True)
    DISPATCHABLELOAD = Column(Numeric, nullable=True)
    NETINTERCHANGE = Column(Numeric, nullable=True)
    EXCESSGENERATION = Column(Numeric, nullable=True)
    LOWER5MINDISPATCH = Column(Numeric, nullable=True)
    LOWER5MINIMPORT = Column(Numeric, nullable=True)
    LOWER5MINLOCALDISPATCH = Column(Numeric, nullable=True)
    LOWER5MINLOCALPRICE = Column(Numeric, nullable=True)
    LOWER5MINLOCALREQ = Column(Numeric, nullable=True)
    LOWER5MINPRICE = Column(Numeric, nullable=True)
    LOWER5MINREQ = Column(Numeric, nullable=True)
    LOWER5MINSUPPLYPRICE = Column(Numeric, nullable=True)
    LOWER60SECDISPATCH = Column(Numeric, nullable=True)
    LOWER60SECIMPORT = Column(Numeric, nullable=True)
    LOWER60SECLOCALDISPATCH = Column(Numeric, nullable=True)
    LOWER60SECLOCALPRICE = Column(Numeric, nullable=True)
    LOWER60SECLOCALREQ = Column(Numeric, nullable=True)
    LOWER60SECPRICE = Column(Numeric, nullable=True)
    LOWER60SECREQ = Column(Numeric, nullable=True)
    LOWER60SECSUPPLYPRICE = Column(Numeric, nullable=True)
    LOWER6SECDISPATCH = Column(Numeric, nullable=True)
    LOWER6SECIMPORT = Column(Numeric, nullable=True)
    LOWER6SECLOCALDISPATCH = Column(Numeric, nullable=True)
    LOWER6SECLOCALPRICE = Column(Numeric, nullable=True)
    LOWER6SECLOCALREQ = Column(Numeric, nullable=True)
    LOWER6SECPRICE = Column(Numeric, nullable=True)
    LOWER6SECREQ = Column(Numeric, nullable=True)
    LOWER6SECSUPPLYPRICE = Column(Numeric, nullable=True)
    RAISE5MINDISPATCH = Column(Numeric, nullable=True)
    RAISE5MINIMPORT = Column(Numeric, nullable=True)
    RAISE5MINLOCALDISPATCH = Column(Numeric, nullable=True)
    RAISE5MINLOCALPRICE = Column(Numeric, nullable=True)
    RAISE5MINLOCALREQ = Column(Numeric, nullable=True)
    RAISE5MINPRICE = Column(Numeric, nullable=True)
    RAISE5MINREQ = Column(Numeric, nullable=True)
    RAISE5MINSUPPLYPRICE = Column(Numeric, nullable=True)
    RAISE60SECDISPATCH = Column(Numeric, nullable=True)
    RAISE60SECIMPORT = Column(Numeric, nullable=True)
    RAISE60SECLOCALDISPATCH = Column(Numeric, nullable=True)
    RAISE60SECLOCALPRICE = Column(Numeric, nullable=True)
    RAISE60SECLOCALREQ = Column(Numeric, nullable=True)
    RAISE60SECPRICE = Column(Numeric, nullable=True)
    RAISE60SECREQ = Column(Numeric, nullable=True)
    RAISE60SECSUPPLYPRICE = Column(Numeric, nullable=True)
    RAISE6SECDISPATCH = Column(Numeric, nullable=True)
    RAISE6SECIMPORT = Column(Numeric, nullable=True)
    RAISE6SECLOCALDISPATCH = Column(Numeric, nullable=True)
    RAISE6SECLOCALPRICE = Column(Numeric, nullable=True)
    RAISE6SECLOCALREQ = Column(Numeric, nullable=True)
    RAISE6SECPRICE = Column(Numeric, nullable=True)
    RAISE6SECREQ = Column(Numeric, nullable=True)
    RAISE6SECSUPPLYPRICE = Column(Numeric, nullable=True)
    AGGEGATEDISPATCHERROR = Column(Numeric, nullable=True)
    AGGREGATEDISPATCHERROR = Column(Numeric, nullable=True)
    LASTCHANGED = Column(Numeric, nullable=True)
    INITIALSUPPLY = Column(Numeric, nullable=True)
    CLEAREDSUPPLY = Column(Numeric, nullable=True)
    LOWERREGIMPORT = Column(Numeric, nullable=True)
    LOWERREGLOCALDISPATCH = Column(Numeric, nullable=True)
    LOWERREGLOCALREQ = Column(Numeric, nullable=True)
    LOWERREGREQ = Column(Numeric, nullable=True)
    RAISEREGIMPORT = Column(Numeric, nullable=True)
    RAISEREGLOCALDISPATCH = Column(Numeric, nullable=True)
    RAISEREGLOCALREQ = Column(Numeric, nullable=True)
    RAISEREGREQ = Column(Numeric, nullable=True)
    RAISE5MINLOCALVIOLATION = Column(Numeric, nullable=True)
    RAISEREGLOCALVIOLATION = Column(Numeric, nullable=True)
    RAISE60SECLOCALVIOLATION = Column(Numeric, nullable=True)
    RAISE6SECLOCALVIOLATION = Column(Numeric, nullable=True)
    LOWER5MINLOCALVIOLATION = Column(Numeric, nullable=True)
    LOWERREGLOCALVIOLATION = Column(Numeric, nullable=True)
    LOWER60SECLOCALVIOLATION = Column(Numeric, nullable=True)
    LOWER6SECLOCALVIOLATION = Column(Numeric, nullable=True)
    RAISE5MINVIOLATION = Column(Numeric, nullable=True)
    RAISEREGVIOLATION = Column(Numeric, nullable=True)
    RAISE60SECVIOLATION = Column(Numeric, nullable=True)
    RAISE6SECVIOLATION = Column(Numeric, nullable=True)
    LOWER5MINVIOLATION = Column(Numeric, nullable=True)
    LOWERREGVIOLATION = Column(Numeric, nullable=True)
    LOWER60SECVIOLATION = Column(Numeric, nullable=True)
    LOWER6SECVIOLATION = Column(Numeric, nullable=True)
    RAISE6SECACTUALAVAILABILITY = Column(Numeric, nullable=True)
    RAISE60SECACTUALAVAILABILITY = Column(Numeric, nullable=True)
    RAISE5MINACTUALAVAILABILITY = Column(Numeric, nullable=True)
    RAISEREGACTUALAVAILABILITY = Column(Numeric, nullable=True)
    LOWER6SECACTUALAVAILABILITY = Column(Numeric, nullable=True)
    LOWER60SECACTUALAVAILABILITY = Column(Numeric, nullable=True)
    LOWER5MINACTUALAVAILABILITY = Column(Numeric, nullable=True)
    LOWERREGACTUALAVAILABILITY = Column(Numeric, nullable=True)
    LORSURPLUS = Column(Numeric, nullable=True)
    LRCSURPLUS = Column(Numeric, nullable=True)
    TOTALINTERMITTENTGENERATION = Column(Numeric, nullable=True)
    DEMAND_AND_NONSCHEDGEN = Column(Numeric, nullable=True)
    UIGF = Column(Numeric, nullable=True)
    SEMISCHEDULE_CLEAREDMW = Column(Numeric, nullable=True)
    SEMISCHEDULE_COMPLIANCEMW = Column(Numeric, nullable=True)


class NemDispatchInterconnectorRes(Base, NemModel):
    __tablename__ = "nem_dispatch_interconnector_res"
    __table_args__ = (
        # Index("nem_dispatch_interconnector_res_uniq", "SETTLEMENTDATE", unique=True,),
    )

    SETTLEMENTDATE = Column(Integer, nullable=False, primary_key=True)
    RUNNO = Column(Integer, nullable=False)
    INTERCONNECTORID = Column(Text, primary_key=True)
    DISPATCHINTERVAL = Column(DateTime(timezone=True), primary_key=True)
    INTERVENTION = Column(Numeric, nullable=True)
    METEREDMWFLOW = Column(Numeric, nullable=True)
    MWFLOW = Column(Numeric, nullable=True)
    MWLOSSES = Column(Numeric, nullable=True)
    MARGINALVALUE = Column(Numeric, nullable=True)
    VIOLATIONDEGREE = Column(Numeric, nullable=True)
    LASTCHANGED = Column(DateTime(timezone=True), nullable=False)
    EXPORTLIMIT = Column(Numeric, nullable=True)
    IMPORTLIMIT = Column(Numeric, nullable=True)
    MARGINALLOSS = Column(Numeric, nullable=True)
    EXPORTGENCONID = Column(Text, nullable=False)
    IMPORTGENCONID = Column(Text, nullable=False)
    FCASEXPORTLIMIT = Column(Numeric, nullable=True)
    FCASIMPORTLIMIT = Column(Numeric, nullable=True)
    LOCAL_PRICE_ADJUSTMENT_EXPORT = Column(Numeric, nullable=True)
    LOCALLY_CONSTRAINED_EXPORT = Column(Numeric, nullable=True)
    LOCAL_PRICE_ADJUSTMENT_IMPORT = Column(Numeric, nullable=True)
    LOCALLY_CONSTRAINED_IMPORT = Column(Numeric, nullable=True)


class NemDispatchConstraint(Base, NemModel):
    __tablename__ = "nem_dispatch_constraint"
    __table_args__ = (
        # Index("nem_dispatch_constraint_uniq", "SETTLEMENTDATE", unique=True,),
    )

    SETTLEMENTDATE = Column(Integer, nullable=False, primary_key=True)
    RUNNO = Column(Integer, nullable=False)

    CONSTRAINTID = Column(Text)
    DISPATCHINTERVAL = Column(DateTime(timezone=True), primary_key=True)
    INTERVENTION = Column(Numeric)
    RHS = Column(Numeric)
    MARGINALVALUE = Column(Integer)
    VIOLATIONDEGREE = Column(Integer)
    LASTCHANGED = Column(DateTime(timezone=True))
    DUID = Column(Text)
    GENCONID_EFFECTIVEDATE = Column(DateTime(timezone=True), nullable=True)
    GENCONID_VERSIONNO = Column(Integer, nullable=False)
    LHS = Column(Numeric, nullable=True)


class NemDispatchInterconnection(Base, NemModel):
    __tablename__ = "nem_dispatch_interconnection"
    __table_args__ = (
        # Index("nem_dispatch_region_sum_uniq", "SETTLEMENTDATE", unique=True,),
    )

    SETTLEMENTDATE = Column(Integer, nullable=False, primary_key=True)
    RUNNO = Column(Integer, nullable=False)


class FuelTech(Base, NemModel):
    __tablename__ = "fueltech"
    code = Column(Text, primary_key=True)
    label = Column(Text, nullable=True)


class WemParticipant(Base, NemModel):
    """

        wem participants

    """

    __tablename__ = "wem_participant"
    code = Column(Text, index=True, primary_key=True)
    name = Column(Text, index=True)


class WemFacility(Base, NemModel):
    """

        wem facility

        http://data.wa.aemo.com.au/#facilities

    """

    __tablename__ = "wem_facility"

    code = Column(Text, primary_key=True)
    participant_id = Column(
        Text,
        ForeignKey("wem_participant.code", name="fk_facility_participant_id"),
    )
    participant = relationship("WemParticipant")

    fueltech_id = Column(
        Text,
        ForeignKey("fueltech.code", name="fk_wem_facility_fueltech_id"),
        nullable=True,
    )
    fueltech = relationship(
        "FuelTech", backref=backref("facilities", cascade="all,delete")
    )

    active = Column(Boolean, default=True)
    capacity_credits = Column(Numeric, nullable=True)
    capacity_maximum = Column(Numeric, nullable=True)
    registered = Column(DateTime)


class WemFacilityScada(Base, NemModel):
    """
        WEM Facility Scada


    """

    __tablename__ = "wem_facility_scada"

    trading_interval = Column(DateTime, index=True, primary_key=True)

    facility_id = Column(
        Text,
        ForeignKey("wem_facility.code", name="fk_facility_scada_facility_id"),
        primary_key=True,
    )
    facility = relationship("WemFacility")

    generated = Column(Numeric, nullable=True)
    eoi_quantity = Column(Numeric, nullable=True)


class WemBalancingSummary(Base, NemModel):

    __tablename__ = "wem_balancing_summary"

    trading_interval = Column(DateTime, index=True, primary_key=True)
    forecast_load = Column(Numeric, nullable=True)
    generation_scheduled = Column(Numeric, nullable=True)
    generation_non_scheduled = Column(Numeric, nullable=True)
    generation_total = Column(Numeric, nullable=True)
    price = Column(Numeric, nullable=True)



import csv
import logging
from datetime import datetime

from scrapy.exceptions import DropItem
from sqlalchemy.exc import IntegrityError
from sqlalchemy.sql import text

from opennem.db.models.wem import WemBalancingSummary
from opennem.pipelines import DatabaseStoreBase
from opennem.utils.pipelines import check_spider_pipeline

logger = logging.getLogger(__name__)


class WemStoreBalancingSummary(DatabaseStoreBase):
    def parse_interval(self, date_str):
        return datetime.strptime(date_str, "%Y-%m-%d %H:%M:%S")

    @check_spider_pipeline
    def process_item(self, item, spider=None):

        s = self.session()

        csvreader = csv.DictReader(item["content"].split("\n"))

        q = self.engine.execute(
            text("select max(trading_interval) from wem_balancing_summary")
        )

        interval_max = q.fetchone()[0] or datetime(1900, 1, 1, 0, 0, 0)

        objects = [
            WemBalancingSummary(
                trading_interval=self.parse_interval(row["Trading Interval"]),
                forecast_load=row["Load Forecast (MW)"],
                generation_scheduled=row["Scheduled Generation (MW)"],
                generation_non_scheduled=row["Non-Scheduled Generation (MW)"],
                generation_total=row["Total Generation (MW)"],
                price=row["Final Price ($/MWh)"],
            )
            for row in csvreader
            if self.parse_interval(row["Trading Interval"]) > interval_max
        ]

        try:
            s.bulk_save_objects(objects)
            s.commit()
        except Exception as e:
            logger.error("Error: {}".format(e))
            raise e
        finally:
            s.close()

        return len(objects)


class WemStoreBalancingSummaryArchive(DatabaseStoreBase):
    def parse_interval(self, date_str):
        return datetime.strptime(date_str, "%Y-%m-%d %H:%M:%S")

    @check_spider_pipeline
    def process_item(self, item, spider=None):

        s = self.session()

        csvreader = csv.DictReader(item["content"].split("\n"))

        q = self.engine.execute(
            text("select distinct trading_interval from wem_balancing_summary")
        )

        intervals_all = [i[0] for i in q.fetchall()]

        objects = [
            WemBalancingSummary(
                trading_interval=self.parse_interval(row["Trading Interval"]),
                forecast_load=row["Load Forecast (MW)"],
                generation_scheduled=row["Scheduled Generation (MW)"],
                generation_non_scheduled=row["Non-Scheduled Generation (MW)"],
                generation_total=row["Total Generation (MW)"],
                price=row["Final Price ($/MWh)"],
            )
            for row in csvreader
            if self.parse_interval(row["Trading Interval"])
            not in intervals_all
        ]

        try:
            s.bulk_save_objects(objects)
            s.commit()
        except Exception as e:
            logger.error("Error: {}".format(e))
            raise e
        finally:
            s.close()

        return len(objects)



import csv
import json
import os
from datetime import date, datetime
from decimal import Decimal

from sqlalchemy.exc import IntegrityError
from sqlalchemy.orm import sessionmaker

from opennem.db import db_connect
from opennem.db.models.bom import BomStation
from opennem.db.models.opennem import FuelTech
from opennem.db.models.wem import metadata

FIXTURE_PATH = os.path.join(os.path.dirname(__file__), "fixtures")

engine = db_connect()
session = sessionmaker(bind=engine)


def load_fixture(fixture_name):
    fixture_path = os.path.join(FIXTURE_PATH, fixture_name)

    if not os.path.isfile(fixture_path):
        raise Exception("Not a file: {}".format(fixture_path))

    fixture = None

    with open(fixture_path) as fh:
        fixture = json.load(fh)

    return fixture


def load_fueltechs():
    fixture = load_fixture("fueltechs.json")

    s = session()

    for fueltech in fixture:
        ft = FuelTech(code=fueltech)

        try:
            s.add(ft)
            s.commit()
        except Exception:
            print("Have {}".format(ft.code))


def parse_date(date_str):
    dt = datetime.strptime(date_str, "%Y%m%d")
    return dt.date()


def parse_fixed_line(line):
    return (
        str(line[:6]),
        str(line[8:11]).strip(),
        str(line[18:59]).strip(),
        parse_date(line[59:67]),
        Decimal(line[75:83]),
        Decimal(line[84:]),
    )


def load_bom_stations():
    s = session()

    with open(os.path.join(FIXTURE_PATH, "stations_db.txt")) as fh:
        lines = fh.readlines()
        for line in lines:
            code, state, name, registered, lat, lng = parse_fixed_line(line)
            station = BomStation(
                code=code,
                state=state,
                name=name,
                registered=registered,
                lat=lat,
                lng=lng,
            )

            try:
                s.add(station)
                s.commit()
            except Exception:
                print("Have {}".format(station.code))


if __name__ == "__main__":
    load_fueltechs()
    load_bom_stations()



from opennem.db import db_connect
from opennem.db.models.bom import metadata as metadata_bom
from opennem.db.models.opennem import WemFacilityScada, metadata

if __name__ == "__main__":
    engine = db_connect()
    metadata.drop_all(engine)
    metadata.create_all(engine)

    metadata_bom.drop_all(engine)
    metadata_bom.create_all(engine)



import csv
import logging
from datetime import datetime

from scrapy.exceptions import DropItem
from sqlalchemy.exc import IntegrityError
from sqlalchemy.sql import text

from opennem.db.models.wem import WemFacilityScada
from opennem.pipelines import DatabaseStoreBase
from opennem.utils.pipelines import check_spider_pipeline

logger = logging.getLogger(__name__)


class WemStoreFacilityScada(DatabaseStoreBase):
    def parse_interval(self, date_str):
        return datetime.strptime(date_str, "%Y-%m-%d %H:%M:%S")

    @check_spider_pipeline
    def process_item(self, item, spider=None):

        s = self.session()

        csvreader = csv.DictReader(item["content"].split("\n"))

        # Get all intervals

        q = self.engine.execute(
            text("select distinct trading_interval from wem_balancing_summary")
        )

        all_intervals = list(set([i[0] for i in q.fetchall()]))

        # Get all facility codes

        q = self.engine.execute(text("select code from wem_facility"))

        all_facilities = list(set([i[0] for i in q.fetchall()]))

        objects = [
            WemFacilityScada(
                trading_interval=self.parse_interval(row["Trading Interval"]),
                facility_id=row["Facility Code"],
                eoi_quantity=row["EOI Quantity (MW)"] or 0,
                generated=row["Energy Generated (MWh)"] or 0,
            )
            for row in csvreader
            if self.parse_interval(row["Trading Interval"])
            not in all_intervals
            and row["Energy Generated (MWh)"] != ""
            and row["Facility Code"] in all_facilities
        ]

        try:
            s.bulk_save_objects(objects)
            s.commit()
        except Exception as e:
            logger.error("Error: {}".format(e))
        finally:
            s.close()

        return len(objects)



import csv
import logging
from datetime import datetime

from scrapy.exceptions import DropItem
from sqlalchemy.exc import IntegrityError

from opennem.db.models.wem import (
    WemBalancingSummary,
    WemFacility,
    WemFacilityScada,
    WemParticipant,
)
from opennem.pipelines import DatabaseStoreBase
from opennem.utils.pipelines import check_spider_pipeline

logger = logging.getLogger(__name__)


class WemStoreFacility(DatabaseStoreBase):
    @check_spider_pipeline
    def process_item(self, item, spider):
        s = self.session()

        if not "Participant Code" in item:
            print("invlid row")
            return item

        participant = None

        participant_code = item["Participant Code"]
        participant = s.query(WemParticipant).get(participant_code)

        if not participant:
            print("Participant not found: {}".format(participant_code))
            participant = WemParticipant(
                code=participant_code, name=item["Participant Name"]
            )
            s.add(participant)
            s.commit()

        facility = None

        facility_code = item["Facility Code"]
        facility = s.query(WemFacility).get(facility_code)

        if not facility:
            facility = WemFacility(
                code=facility_code, participant=participant,
            )

        facility.active = (
            False if item["Balancing Status"] == "Non-Active" else True
        )

        if item["Capacity Credits (MW)"]:
            facility.capacity_credits = item["Capacity Credits (MW)"]

        if item["Maximum Capacity (MW)"]:
            facility.capacity_maximum = item["Maximum Capacity (MW)"]

        registered_date = item["Registered From"]

        if registered_date:
            registered_date_dt = datetime.strptime(
                registered_date, "%Y-%m-%d %H:%M:%S"
            )
            facility.registered = registered_date_dt

        try:
            s.add(facility)
            s.commit()
        except IntegrityError as e:
            pass
        except Exception as e:
            logger.error("Error: {}".format(e))
        finally:
            s.close()

        return item


class WemStoreFacilityScada(DatabaseStoreBase):
    @check_spider_pipeline
    def process_item(self, item, spider=None):

        s = self.session()

        if not "Participant Code" in item:
            print("invlid row")
            return item

        trading_interval = item["Trading Interval"]

        trading_interval_dt = datetime.strptime(
            trading_interval, "%Y-%m-%d %H:%M:%S"
        )

        # @TODO we possibly need to shift eoi_quantity back a time period

        facility_scada = WemFacilityScada(
            trading_interval=trading_interval_dt,
            facility_id=item["Facility Code"],
            eoi_quantity=item["EOI Quantity (MW)"],
            generated=item["Energy Generated (MWh)"],
        )

        try:
            s.add(facility_scada)
            s.commit()
        except IntegrityError as e:
            pass
        except Exception as e:
            logger.error("Error: {}".format(e))
        finally:
            s.close()

        return item



import csv

from scrapy import Spider

from opennem.pipelines.wem import WemStoreFacility


class WemFacilities(Spider):
    name = "au.wem.facilities"
    start_urls = [
        "http://data.wa.aemo.com.au/datafiles/facilities/facilities.csv"
    ]
    # allowed_domains = ["wa.nemweb.com.au"]
    pipelines = set([WemStoreFacility,])

    def parse(self, response):
        csvreader = csv.DictReader(response.text.split("\n"))
        for row in csvreader:
            yield row



from huey import RedisHuey, crontab

from opennem.api.exporter import wem_run_all
from opennem.settings import get_redis_host

REDIS_HOST = get_redis_host()

huey = RedisHuey("opennem.exporter", host=REDIS_HOST)


@huey.periodic_task(crontab(minute="*/10"))
def wem_export_task():
    wem_run_all()



import csv

import scrapy

from opennem.pipelines.wem.facility_scada import WemStoreFacilityScada
from opennem.spiders.wem import WemCurrentSpider, WemHistoricSpider


class WemCurrentFacilityScada(WemCurrentSpider):
    name = "au.wem.current.facility_scada"
    start_url = "http://data.wa.aemo.com.au/public/public-data/datafiles/facility-scada/facility-scada-{year}-{month}.csv"

    pipelines_extra = set([WemStoreFacilityScada])


class WemHistoricFacilityScada(WemHistoricSpider):
    name = "au.wem.historic.facility_scada"
    start_urls = ["http://data.wa.aemo.com.au/datafiles/facility-scada/"]

    pipelines_extra = set([WemStoreFacilityScada])



import json
import logging

import scrapy

from opennem.pipelines.bom import StoreBomObservation


class BomJSONObservationSpider(scrapy.Spider):
    allowed_domains = ["bom.gov.au"]
    start_urls = ["http://reg.bom.gov.au/fwo/IDN60901/IDN60901.94767.json"]
    pipelines = set([StoreBomObservation])

    def parse(self, response):
        json_response = json.loads(response.text)

        if not type(json_response) is dict:
            raise Exception("invalid response from bom")

        if not "observations" in json_response:
            raise Exception("invalid response from bom")

        if not "data" in json_response["observations"]:
            raise Exception("invalid response from bom")

        for i in json_response["observations"]["data"]:
            yield i



import datetime
import os
import re
import time
from io import BytesIO, StringIO

import pandas as pd
import requests
from sqlalchemy.exc import IntegrityError

from dms_daemon import CONFIG, live, orm
from dms_daemon.market_notice_reader import check_notice
from dms_daemon.nemweb_reader import NemFile, ZipFileStreamer


def slack_dms_webhook(message):
    json_text = "{0} \n ({1})".format(
        message, datetime.datetime.now().strftime("%Y-%m-%d %H:%M")
    )
    requests.post(
        "https://hooks.slack.com/services/{0}".format(
            CONFIG["slack_hooks"]["error_message"]
        ),
        json={"text": "{0}".format(json_text)},
    )


def get_page(link="http://www.nemweb.com.au/Reports/CURRENT/Dispatch_SCADA/"):
    return requests.get(link)


def get_zip(
    href="/Reports/CURRENT/Dispatch_SCADA/PUBLIC_DISPATCHSCADA_201708161600_0000000286435695.zip",
):
    response = requests.get("http://www.nemweb.com.au{0}".format(href))
    return BytesIO(response.content)


def last_date(table="DISPATCH_UNIT_SCADA", column="SETTLEMENTDATE", db="nemweb_live"):
    cursor = orm.engine2.execute("SELECT MAX({1}) FROM {0}".format(table, column))
    return cursor.fetchall()[0][0]


def process_zip(link):
    zipIO = get_zip(href=link)
    scada_file = live.load_scadafile(zipIO)
    return scada_file


def insert(df, name):
    for key in orm.keys.keys():
        if key in df.columns:
            # df[key] = df[key].apply(lambda x: orm.keys[key][x])
            try:
                df[key] = df[key].apply(lambda x: orm.keys[key][x])
            except KeyError as e:
                if (key == "DUID") & (name == "DISPATCH_UNIT_SCADA"):
                    orm.engine_meta.execute(
                        "INSERT INTO DUID (DUID) VALUES ({0})".format(e)
                    )
                else:
                    raise e

    table = orm.Base.classes[name]
    columns = [col.name for col in table.__table__.columns]

    if "ROP" in columns:
        columns.remove("ROP")

    dx = df[columns[1:]]
    dx.to_sql(name, con=orm.engine2, if_exists="append", index=None)


# Instances of live inserters


def dispatch_scada():
    d = last_date()
    page = get_page()
    tables = ["DISPATCH_UNIT_SCADA"]

    pattern = re.compile(
        "/Reports/CURRENT/Dispatch_SCADA/PUBLIC_DISPATCHSCADA_([0-9]{12})_[0-9]{16}.zip"
    )

    for match in pattern.finditer(page.text):
        dt = datetime.datetime.strptime(match.group(1), "%Y%m%d%H%M")
        if dt > d:
            nemfile = process_zip(match.group(0))

            for table in tables:
                df = nemfile.__dict__[table].drop_duplicates().copy()
                insert(df, table)
                print(table, dt)


def rooftop_actual():
    d = last_date(table="ROOFTOP_ACTUAL", column="INTERVAL_DATETIME")
    page = get_page(link="http://www.nemweb.com.au/Reports/Current/ROOFTOP_PV/ACTUAL/")
    tables = ["ROOFTOP_ACTUAL"]

    pattern = re.compile(
        "/Reports/Current/ROOFTOP_PV/ACTUAL/PUBLIC_ROOFTOP_PV_ACTUAL_MEASUREMENT_([0-9]{14})_[0-9]{16}.zip"
    )

    for match in pattern.finditer(page.text):
        dt = datetime.datetime.strptime(match.group(1), "%Y%m%d%H%M00")
        if dt > d + datetime.timedelta(0, 1800):
            # print (match.group(0))
            nemfile = process_zip(match.group(0))
            nemfile.ROOFTOP_ACTUAL = nemfile.ROOFTOP_ACTUAL[
                nemfile.ROOFTOP_ACTUAL.REGIONID.isin(
                    ["NSW1", "QLD1", "SA1", "TAS1", "VIC1"]
                )
            ]

            for table in tables:
                df = nemfile.__dict__[table].drop_duplicates().copy()
                insert(df, table)
                print(table, dt)


def dispatch_unit_solution():
    d = last_date(table="DISPATCH_UNIT_SOLUTION")
    page = get_page(link="http://nemweb.com.au/Reports/Current/Next_Day_Dispatch/")
    tables = ["DISPATCH_UNIT_SOLUTION"]

    pattern = re.compile(
        "/Reports/Current/Next_Day_Dispatch/PUBLIC_NEXT_DAY_DISPATCH_([0-9]{8})_[0-9]{16}.zip"
    )

    for match in pattern.finditer(page.text):
        dt = datetime.datetime.strptime(match.group(1), "%Y%m%d")
        if (
            dt.date() >= d.date()
        ):  # not that same as metered gen - should probably make them consistent
            nemfile = process_zip(match.group(0))

            for table in tables:
                df = nemfile.__dict__[table].drop_duplicates().copy()
                insert(df, table)
                print(table, dt)


def dispatch_unit_duplicate(
    link="http://www.nemweb.com.au/Reports/Current/Next_Day_Dispatch/DUPLICATE/",
):
    d = datetime.datetime(2020, 4, 1)
    page = get_page(link)
    tables = ["DISPATCH_UNIT_SOLUTION"]

    pattern = re.compile(
        "/Reports/Current/Next_Day_Dispatch/DUPLICATE/PUBLIC_NEXT_DAY_DISPATCH_([0-9]{8})_[0-9]{16}.zip"
    )

    for match in pattern.finditer(page.text):
        dt = datetime.datetime.strptime(match.group(1), "%Y%m%d")
        if (
            dt.date() >= d.date()
        ):  # not that same as metered gen - should probably make them consistent
            print(dt)
            nemfile = process_zip(match.group(0))

            for table in tables:
                df = nemfile.__dict__[table].drop_duplicates().copy()
                insert(df, table)
                print(table, dt)


def metered_data_gen_duid():
    d = last_date(table="METER_DATA_GEN_DUID", column="INTERVAL_DATETIME")
    last_trading_day = datetime.datetime(d.year, d.month, d.day) - datetime.timedelta(1)

    page = get_page(link="http://www.nemweb.com.au/Reports/Current/Next_Day_Actual_Gen/")
    tables = ["METER_DATA_GEN_DUID"]

    pattern = re.compile(
        "/Reports/Current/Next_Day_Actual_Gen/PUBLIC_NEXT_DAY_ACTUAL_GEN_([0-9]{8})_[0-9]{16}.zip"
    )

    for match in pattern.finditer(page.text):
        dt = datetime.datetime.strptime(match.group(1), "%Y%m%d")
        if dt > last_trading_day:
            nemfile = process_zip(match.group(0))

            for table in tables:
                df = nemfile.__dict__[table].drop_duplicates().copy()
                insert(df, table)
                print(table, dt)


def rooftop_forecast(d):
    page = get_page(link="http://www.nemweb.com.au/Reports/Current/ROOFTOP_PV/FORECAST/")
    tables = ["ROOFTOP_FORECAST"]

    pattern = re.compile(
        "/Reports/Current/ROOFTOP_PV/FORECAST/PUBLIC_ROOFTOP_PV_FORECAST_([0-9]{14})_[0-9]{16}.zip"
    )

    for match in pattern.finditer(page.text):
        dt = datetime.datetime.strptime(match.group(1), "%Y%m%d%H%M00")
        if dt > d:
            nemfile = process_zip(match.group(0))

            for table in tables:
                df = nemfile.__dict__[table].drop_duplicates().copy()
                insert(df, table)
                print(table, dt)


def rooftop_forecast_duplicate():
    d = last_date(table="ROOFTOP_FORECAST", column="VERSION_DATETIME")
    try:
        rooftop_forecast(d)
    except IntegrityError as E:
        d = datetime.datetime.strptime(E.params[0][0], "%Y/%m/%d %H:%M:%S")
        slack_dms_webhook("Duplicate forecast `{0}`\n{1}".format(d, E.args[0]))
        rooftop_forecast(d + datetime.timedelta(0, 1800))


def dispatch_is():
    d = last_date(table="DISPATCH_PRICE")
    page = get_page(link="http://www.nemweb.com.au/Reports/CURRENT/DispatchIS_Reports/")
    tables = ["DISPATCH_PRICE", "DISPATCH_REGIONSUM", "DISPATCH_INTERCONNECTORRES"]

    pattern = re.compile(
        "/Reports/CURRENT/DispatchIS_Reports/PUBLIC_DISPATCHIS_([0-9]{12})_[0-9]{16}.zip"
    )

    for match in pattern.finditer(page.text):
        dt = datetime.datetime.strptime(match.group(1), "%Y%m%d%H%M")
        if dt > d:
            nemfile = process_zip(match.group(0))

            for table in tables:
                df = nemfile.__dict__[table].drop_duplicates().copy()
                insert(df, table)
                print(table, dt)


def trading_is():
    d = last_date(table="TRADING_PRICE")
    page = get_page(link="http://www.nemweb.com.au/Reports/CURRENT/TradingIS_Reports/")
    tables = ["TRADING_PRICE", "TRADING_REGIONSUM", "TRADING_INTERCONNECTORRES"]

    pattern = re.compile(
        "/Reports/CURRENT/TradingIS_Reports/PUBLIC_TRADINGIS_([0-9]{12})_[0-9]{16}.zip"
    )

    for match in pattern.finditer(page.text):
        dt = datetime.datetime.strptime(match.group(1), "%Y%m%d%H%M")
        if dt > d:
            nemfile = process_zip(match.group(0))

            for table in tables:
                df = nemfile.__dict__[table].drop_duplicates().copy()
                if "ROP" in df.columns:
                    df = df.drop("ROP", axis=1)
                insert(df, table)
                print(table, dt)


def bidoffer_energy():
    d = last_date(table="OFFER_BIDPEROFFER")
    page = get_page(
        link="http://www.nemweb.com.au/Reports/CURRENT/Next_Day_Offer_Energy/"
    )
    tables = ["OFFER_BIDDAYOFFER", "OFFER_BIDPEROFFER"]

    pattern = re.compile(
        "/Reports/CURRENT/Next_Day_Offer_Energy/PUBLIC_NEXT_DAY_OFFER_ENERGY_([0-9]{8})_[0-9]{16}.zip"
    )

    for match in pattern.finditer(page.text):
        dt = datetime.datetime.strptime(match.group(1), "%Y%m%d")
        if dt > d:
            nemfile = process_zip(match.group(0))

            for table in tables:
                df = nemfile.__dict__[table].drop_duplicates().copy()
                insert(df, table)
                print(table, dt)


def bidoffer_fcas():
    d = datetime.datetime(2017, 7, 31)
    page = get_page(link="http://www.nemweb.com.au/Reports/CURRENT/Next_Day_Offer_FCAS/")
    tables = ["OFFER_BIDDAYOFFER", "OFFER_BIDPEROFFER"]

    pattern = re.compile(
        "/Reports/CURRENT/Next_Day_Offer_FCAS/PUBLIC_NEXT_DAY_OFFER_FCAS_([0-9]{8})_[0-9]{16}.zip"
    )

    for match in pattern.finditer(page.text):
        dt = datetime.datetime.strptime(match.group(1), "%Y%m%d")
        if dt > d:
            nemfile = process_zip(match.group(0))

            for table in tables:
                df = nemfile.__dict__[table].drop_duplicates().copy()
                insert(df, table)
                print(table, dt)


def seqno_to_dt(x):
    x = str(x)
    year = int(x[:4])
    month = int(x[4:6])
    day = int(x[6:8])
    ti = int(x[-2:])
    return datetime.datetime(year, month, day, 4) + ti * datetime.timedelta(0, 1800)


def predispatch():
    d = last_date(table="PREDISPATCH_REGION_SOLUTION", column="PREDISPATCHSEQNO")
    # d = datetime.datetime(2020,1,30)
    d = seqno_to_dt(d)
    page = get_page(
        link="http://www.nemweb.com.au/Reports/CURRENT/PredispatchIS_Reports/"
    )
    tables = ["PREDISPATCH_REGION_SOLUTION", "PREDISPATCH_REGION_PRICES"]

    pattern = re.compile(
        "/Reports/CURRENT/PredispatchIS_Reports/PUBLIC_PREDISPATCHIS_([0-9]{12})_[0-9]{14}.zip"
    )

    for match in pattern.finditer(page.text):
        dt = datetime.datetime.strptime(match.group(1), "%Y%m%d%H%M")
        if dt > d:
            nemfile = process_zip(match.group(0))

            for table in tables:
                df = nemfile.__dict__[table].drop_duplicates().copy()
                insert(df, table)
                print(table, dt)


def market_notices():
    page = get_page(link="http://www.nemweb.com.au/Reports/Current/Market_Notice/")

    pattern = re.compile(
        "/Reports/Current/Market_Notice/NEMITWEB1_MKTNOTICE_([0-9]{8}).R([0-9]{5})"
    )

    for match in pattern.finditer(page.text):
        dt = datetime.datetime.strptime(match.group(1), "%Y%m%d")
        notice_dir = "/data/marble/nemweb/MARKET_NOTICES/{0}/".format(
            datetime.date(dt.year, dt.month, dt.day)
        )

        if os.path.isdir(notice_dir) == False:
            os.mkdir(notice_dir)

        filename = "{0}{1}.txt".format(notice_dir, match.group(2))

        if os.path.exists(filename):
            continue

        else:
            print(filename)
            response = requests.get("http://www.nemweb.com.au{0}".format(match.group(0)))
            with open(filename, "wb") as f:
                f.write(response.content)
            check_notice(filename)


if __name__ == "__main__":
    for func in [
        trading_is,
        dispatch_is,
        dispatch_scada,
        market_notices,
        rooftop_actual,
        rooftop_forecast_duplicate,
    ]:
        try:
            func()
        except Exception as E:
            print(func)
            slack_dms_webhook(
                "`{0}` failed from file `{1}`\n{2}\n{3}".format(
                    func.__name__, __file__, E, E.args
                )
            )



from opennem.db.models.opennem import (
    Base,
    WemBalancingSummary,
    WemFacility,
    WemFacilityScada,
    WemParticipant,
    metadata,
)

if __name__ == "__main__":
    from opennem.db import db_connect

    engine = db_connect()
    metadata.drop_all(engine)
    metadata.create_all(engine)
